"""
Copyright 2009, 2010, 2011 Leif Theden

This file is part of lib2d.

lib2d is free software: you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from lib2d.gamestate import GameState
from lib2d.cmenu import cMenu
from lib2d.statedriver import driver as sd
from lib2d.renderer import Camera
from lib2d.banner import TextBanner
from lib2d import param
import lib2d.gfx as gfx

from mob import Monster

from random import randint
import sys, os.path

from pygame.rect import Rect
import pygame.draw


class BattleState(GameState):
    def __init__(self, monster0, monster1):
        self.menu = cMenu(Rect((10,160),sd.get_size()),
            20, 0, 'vertical', 100,
            [('Attack', self.attack),
            ('Magic', self.use_magic),
            ('Item', self.use_item),
            ('Flee', self.flee)],
            font="fonts/dpcomic.ttf", font_size=16,
            banner_style="normal")

        m0 = Monster()
        m0.render()

        self.monsters = []
        self.monsters.append(m0)

        # for transition effects
        self.state = 0
        self.counter = 0
        self.bkg_color = (0,0,0)

    def update(self, time):
        self.menu.update(time)

    def handle_event(self, event):
        self.menu.handle_event(event)

    def activate(self):
        GameState.activate(self)
        self.menu.ready()
        self.reactivate()

    def reactivate(self):
        self.state = 0
        self.counter = 0

    def deactivate(self):
        pass

    def draw(self, surface):
        if self.state == 0:
            surface.lock()
            for y in range(0, 240, 2):
                x = randint(-10, 10) + (self.counter * 20)
                pygame.draw.line(surface, self.bkg_color, (0,y), (x, y), 2)
            surface.unlock()

            self.counter += 1

            if self.counter >= 17:
                self.state = 1
                self.counter = 0

        elif self.state == 1:
            self.state = 2

        else:
            sd.get_screen().fill(self.bkg_color)
            surface.blit(self.monsters[0].image, (20,60))
            self.menu.draw(surface)

    def attack(self):
        m0 = Monster()
        m0.render()
        self.monsters = []
        self.monsters.append(m0) 

    def use_magic(self):
        self.monsters[0].mDNA.mutate()
        self.monsters[0].render()

    def use_item(self):
        pass

    def flee(self):
        sd.done() 


