"""
Copyright 2009, 2010, 2011 Leif Theden

This file is part of lib2d.

lib2d is free software: you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from lib2d.gamestate import GameState
from lib2d.cmenu import cMenu
from lib2d.tilemap import OneTileMap, View
from lib2d.cutscene import Cutscene
from lib2d.statedriver import driver as sd
from lib2d.banner import TextBanner
from lib2d import param
import lib2d.gfx as gfx
import lib2d.sfx as sfx

from pygame.rect import Rect
import pygame

from worldstate import WorldState
from battlestate import BattleState
from introstate import IntroState

from maps import VillageMap

import os.path

class TitleScreen(GameState):
    def __init__(self):
        GameState.__init__(self)

        self.menu = cMenu(Rect((83,40),sd.get_size()),
            20, 5, 'vertical', 100,
            [('New Game', self.new_game),
            ('Battle Test', self.continue_game),
            ('Introduction', self.show_intro),
            ('Quit Game', self.quit_game)],
            font="fonts/northwoodhigh.ttf", font_size=32)

        self.black = pygame.Surface(sd.get_size())
        self.black.set_alpha(10)
        self.next_state = None

        self.state = 0
        self.counter = 0

    def prepare_background(self):
        """
        subpixel rendering is experimental, but demo'd in the clouds
        notice they don't "snap" to the next pixel, there is very smooth
        movement, but requires a ton of memory and is slow to prepare

        we prepare the cloud layer here, after a wait message has been posted
        """

        clouds = OneTileMap("misc/clouds_1.gif")
        self.background = View(clouds, sd.get_size(), subpixel=True) 

    def update(self, time):

        if self.state == 1:
            self.prepare_background()
            self.state = 2

        elif self.state == 2:
            self.background.update(time)
            self.background.scroll((.1, 0.0))
            self.menu.update(time)

    def handle_event(self, event):
        if self.next_state == None:
            self.menu.handle_event(event)

    def activate(self):
        sfx.fadeout_music()
        GameState.activate(self)
        self.menu.ready()

    def draw(self, surface):
        if self.state == 0:
            surface.fill((0,0,0))
            msg = TextBanner("Please wait...", (128, 128, 128), size=18)
            surface.blit(msg.render(alpha=True), (200,210))
            self.state = 1
            return

        if self.next_state != None:
            surface.blit(self.black, (0,0))
            self.counter += 1
            if self.counter == 70:
                self.black.set_alpha(255)
                surface.blit(self.black, (0,0))
                sd.start_restart(self.next_state)
                pygame.time.wait(2400)
        else:
            self.background.draw(surface)
            self.menu.draw(surface)

    def new_game(self):
        self.next_state = VillageMap()

    def continue_game(self):
        sd.start_restart(BattleState(None, None))

    def show_intro(self):
        self.next_state = \
            Cutscene(os.path.join(param.resource_path, "scripts", "intro.txt")) 

    def quit_game(self):
       sd.done() 

