"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from buttons import *
from banner import TextBanner, OutlineTextBanner
from cmenu import cMenu 
from gamestate import GameState
from statedriver import driver as sd
import gfx, sfx

from pygame.locals import *
from pygame.surface import Surface
import pygame.draw as draw

from textwrap import wrap

class TextDialog(GameState):
    #wrap_width = 58 # font 12
    
    text_size = 14
    wrap_width = 46 
 
    background = (128, 128, 128)

    wait_sound = sfx.load("select0.wav")
    wait_sound.set_volume(0.20)

    def __init__(self, text, title=None):
        GameState.__init__(self)
        self.text = text
        self.state = 0
        self.counter = 0
        self.title = title

    # when given focus for first time
    def activate(self):
        xsize = 300
        ysize = 70
        bkg = Surface((xsize, ysize))
        bkg.lock()
        bkg.fill((128,128,128))
        for i in range(1, 4):
            draw.rect(bkg, (i*32, i*32, i*32), (4-i, 4-i, xsize+(i-4)*2, ysize+(i-4)*2), 3)

        corner = (64,64,64)
        bkg.set_at((0,0), corner)
        bkg.set_at((xsize,0), corner)
        bkg.set_at((xsize,ysize), corner)
        bkg.set_at((0,ysize), corner)

        bkg.set_alpha(64)
        
        bkg.unlock()

        self.bkg = bkg

        if self.title != None:
            banner = OutlineTextBanner(self.title, (200,200,200), 20)
            self.title_image = banner.render()
            self.title_image.set_alpha(96)

        self.arrow = gfx.load_colorkey("misc/wait_arrow.png")

    # when focus is given again
    def reactivate(self):
        pass

    # when losing focus
    def deactivate(self):
        pass

    def draw(self, surface):
        # fade in the dialog box background
        if self.state == 0:
            surface.blit(self.bkg, (10,160))
            self.counter += 1
            if self.counter == 6:
                self.counter = 0
                self.state = 1
                self.bkg = None

        # fade in the title, if any
        elif self.state == 1:
            if self.title != None:
                surface.blit(self.title_image, (15,150))
                self.counter += 1
                if self.counter == 3:
                    self.state = 2
                    self.counter = 0
                    self.title_image = None
            else:
                self.state = 2

        # quickly write the text
        elif self.state == 2:
            x = 20

            if self.title != None:
                y = 168
            else:
                y = 167

            for line in wrap(self.text, self.wrap_width):
                banner = TextBanner(line, size=self.text_size)
                surface.blit(banner.render(self.background), (x,y))
                y += banner.font.size(line)[1]

            self.state = 3

        elif self.state == 3:
            surface.blit(self.arrow, (20, 220))
            self.wait_sound.stop()
            self.wait_sound.play()
            self.state = 4

    def handle_event(self, event):
        if self.state >= 2:
            if event.type == KEYDOWN:
                sd.done()

    def handle_commandlist(self, cmdlist):
        for cls, cmd, arg in cmdlist:
            if (cmd == P1_ACTION1) and arg:
                sd.done()   

    def update(self, time):
        pass


class ChoiceDialog(GameState):
    #wrap_width = 58 # font 12
    
    text_size = 14
    wrap_width = 46 
 
    background = (128, 128, 128)

    wait_sound = sfx.load("select0.wav")
    wait_sound.set_volume(0.20)

    def __init__(self, text, choices, title=None):
        GameState.__init__(self)
        self.text = text
        self.state = 0
        self.counter = 0
        self.title = title
        self.choices = choices

    # when given focus for first time
    def activate(self):
        xsize = 300
        ysize = 70
        bkg = Surface((xsize, ysize))
        bkg.lock()
        bkg.fill((128,128,128))
        for i in range(1, 4):
            draw.rect(bkg, (i*32, i*32, i*32), (4-i, 4-i, xsize+(i-4)*2, ysize+(i-4)*2), 3)

        corner = (64,64,64)
        bkg.set_at((0,0), corner)
        bkg.set_at((xsize,0), corner)
        bkg.set_at((xsize,ysize), corner)
        bkg.set_at((0,ysize), corner)

        bkg.set_alpha(64)
        
        bkg.unlock()

        self.bkg = bkg

        if self.title != None:
            banner = OutlineTextBanner(self.title, (200,200,200), 20)
            self.title_image = banner.render()
            self.title_image.set_alpha(96)

        self.arrow = gfx.load_colorkey("misc/wait_arrow.png")

    # when focus is given again
    def reactivate(self):
        pass

    # when losing focus
    def deactivate(self):
        pass

    def draw(self, surface):
        # fade in the dialog box background
        if self.state == 0:
            surface.blit(self.bkg, (10,160))
            self.counter += 1
            if self.counter == 6:
                self.counter = 0
                self.state = 1
                self.bkg = None

        # fade in the title, if any
        elif self.state == 1:
            if self.title != None:
                surface.blit(self.title_image, (15,150))
                self.counter += 1
                if self.counter == 3:
                    self.state = 2
                    self.counter = 0
                    self.title_image = None
            else:
                self.state = 2

        # quickly write the text
        elif self.state == 2:
            x = 20

            if self.title != None:
                y = 168
            else:
                y = 167

            for line in wrap(self.text, self.wrap_width):
                banner = TextBanner(line, size=self.text_size)
                surface.blit(banner.render(self.background), (x,y))
                y += banner.font.size(line)[1]

            self.menu = cMenu(Rect((25,210),(280, 30)),
                5, 5, 'horizontal', 10,
                [('Yes', self.yes),
                ('No', self.no)],
                font="fonts/dpcomic.ttf", font_size=16)

            self.state = 3

        elif self.state == 3:
            self.menu.draw(surface)

    def handle_event(self, event):
        if self.state > 2:
            self.menu.handle_event(event)

    def handle_commandlist(self, cmdlist):
        for cls, cmd, arg in cmdlist:
            if (cmd == P1_ACTION1) and arg:
                sd.done()   

    def update(self, time):
        pass

    def choice0(self): pass
    def choice1(self): pass
    def choice2(self): pass
    def choice3(self): pass

    def yes(self): sd.done()
    def no(self): sd.done()
