"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from vec import Vec2d
import param

from pygame.transform import scale, scale2x
from pygame.display import flip
import pygame, os.path



"""
loading images
converting for good blits
caching stuff
managing screen buffer

screen can be optionally "pixelized" or scaled.
when this happens, all of the sprites
that were created for the old display
will no longer be the same format
as the display.  huh.

"""

DEBUG = False

def debug(text):
    if DEBUG: sys.stdout.write(text)

image_cache      = []
image_path_cache = {}

def add_image_to_cache(image, fullpath=None):
    global image_cache
    global image_path_cache

    image_cache.append(image)
    if fullpath != None:
        image_path_cache[fullpath] = image_cache.index(image)

# we can pass an image here to see if there is something
# like it already in memory

# BUG: cache lookups don't work
def get_image_from_cache(fullpath=None, image=None):
    global image_cache
    global image_path_cache

    try:
        return image_cache[image_path_cache[fullpath]]
    except KeyError:
        pass

    # only way i know how to compare images.  ok?
    if image != None:
        for cached_image in image_cache:
            if cached_image == image:
                print "from cache"
                return cached_image

    return None

def load(name, use_cache=True, convert=True):
    global image_cache

    fullpath = os.path.join(param.resource_path, name)

    if use_cache:
        image = get_image_from_cache(fullpath)
        if image != None: return image

    try:
        image = pygame.image.load(fullpath)
        if convert: image = image.convert()
    except pygame.error, message:
        print 'Cannot load image:', name
        raise SystemExit, message

    if use_cache:
        add_image_to_cache(image, fullpath)

    return image

def load_image_with_alpha(name, use_cache=True):
    image = load(name, False, False)
    image = image.convert_alpha()
    return image

def load_colorkey(name, use_cache=True):
    image = load(name, use_cache)

    # by default, use the color at the upper left corner at the colorkey
    image.set_colorkey(image.get_at((0,0)))

    return image


pixelize = None
pix_scale = 1
screen_dim = None
buffer_dim = None
screen = None
screen_surface = None
update_display = None

def init(pix=False):
    global screen_dim

    pygame.display.set_caption("RPG World Test")


# is it redundant to have a pygame buffer, and one for pixalization?  maybe...

def update_display_scaled2x():
    scale2x(screen, screen_surface)
    flip()

def update_display_scaled():
    scale(screen, screen_dim, screen_surface)
    flip()

def set_screen(dim, scale=1, transform=None):

    global pixelize, pix_scale, buffer_dim, screen, update_display, screen_surface, screen_dim

    screen_dim = Vec2d(dim)

    if transform == "scale2x" or transform == "scale":
        set_scale(scale, transform)

    elif transform == None:
        pixelize = False
        pixel_buffer = None
        pix_scale = 1
        buffer_dim = None
        update_display = flip
        screen_surface = pygame.display.set_mode(screen_dim, pygame.DOUBLEBUF | pygame.HWSURFACE)
        screen = screen_surface

def set_scale(scale, transform="scale"):
    from pygame.surface import Surface

    global pixelize, pix_scale, buffer_dim, screen, update_display, screen_surface, screen_dim

    if transform == "scale2x":
        pix_scale = 2
        update_display = update_display_scaled2x
    elif transform == "scale":
        pix_scale = scale
        update_display = update_display_scaled

    pixelize = True
    buffer_dim = tuple([ int(x / pix_scale) for x in screen_dim ])
    screen_surface = pygame.display.set_mode(screen_dim)
    screen = Surface(buffer_dim)
    #screen_surface = pygame.display.set_mode(screen_dim)

"""
# for sweet transistions!
def fadeout(time):
    time = float(time)
    oldx = 0
    start = pygame.get_ticks()
    while x < screen_dim[0]
        x = int((pygame.get_ticks()-start) / time * screen_dim[0])
        surface.fill((0,0,0), (0,0,x,screen_dim[1]))
        oldx = x
"""

