"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from rpg import Actor
from tilemap import View
from lib2d import UpdateableObject, ObjectHandler, DrawableObject
from pygame.surface import Surface
from pygame.rect import Rect



class Camera(object):
    """
    draws a tile layer and sprites, etc
    """

    def __init__(self, object_group, tilemap, extent=None):
        self.object_group = object_group.actors
        self.zoom = 1.0
        self.tilemap = tilemap
        
        # HACK
        self.extent = Rect(extent)

    def update(self, time):
        self.tilemap.update(time)

    def draw(self, surface):
        # WEEEEE!!!!!

        self.tilemap.draw_layer_no(surface, 0)
        self.tilemap.draw_layer_no(surface, 1)

        # the sprites should be sorted by position on screen
        # the sprites in the back need to be drawn first
        on_screen = []
        for a in self.object_group:
            pos = self.toSurface(a.position)
            if self.extent.collidepoint(a.position):
                on_screen.append((pos[1], a.avatar.image, pos))
       
        on_screen.sort() 
        [ surface.blit(a[1], a[2]) for a in on_screen ] 

        self.tilemap.draw_layer_no(surface, 2)

    def center(self, pos):
        self.extent.center = (pos[0], pos[1])
        self.tilemap.center(pos)

    def toSurface(self, pos):
        """
        Transform the position to coordinates on the surface
        """

        return (pos[0] * self.zoom) - self.extent.left, (pos[1] * self.zoom) - self.extent.top

    def clear(self, surface):
        raise NotImplementedError

    def add(self, obj):
        if isinstance(obj, Actor):
            self.avatars.append(obj)
            return True
        else:
            return False

